#ifndef VECTOR_MATH_H_INCLUDED
#define VECTOR_MATH_H_INCLUDED

#include <math.h>
#include <memory.h>

#ifndef sqrtf
#define sqrtf(a) ((float)sqrt(a))
#endif

// -------------------------------------------------------------- //
// 3D Globals
// -------------------------------------------------------------- //
extern float			LastVectors[3][3]; // [ID][XYZ]

// -------------------------------------------------------------- //
// gGetLength: Get Length of Vetrex
// -------------------------------------------------------------- //
inline static float gGetLength(float* vertex)
{
	// calculate the length of the normal
	return sqrtf((vertex[0])*(vertex[0]) + 
				 (vertex[1])*(vertex[1]) + 
				 (vertex[2])*(vertex[2]));
}

// -------------------------------------------------------------- //
// gAddVertex: Add one vertex to another
// -------------------------------------------------------------- //
inline static void gAddVertex(float* v1, float* v2)
{
	v1[0]+=v2[0];
	v1[1]+=v2[1];
	v1[2]+=v2[2];
}

// -------------------------------------------------------------- //
// gAddVertex: Add one vertex to another
// -------------------------------------------------------------- //
inline static void gMultVertex(float* v1, float scale)
{
	v1[0]*=scale;
	v1[1]*=scale;
	v1[2]*=scale;
}

// -------------------------------------------------------------- //
// gUnify: Make unit vector
// -------------------------------------------------------------- //
inline static void gUnify(float* v1)
{
float	len = gGetLength(v1);
	if (len!=0.0f)
	{
		v1[0]/=len;
		v1[1]/=len;
		v1[2]/=len;
	}
}

// -------------------------------------------------------------- //
// gInvertMatrix4x4: Invert a 4x4 matrix
// -------------------------------------------------------------- //
inline static void gInvertMatrix4x4(float m[16])
{
static float temp[16];
	temp[0]=m[0];
	temp[1]=m[4];
	temp[2]=m[8];
	temp[3]=0.0f;
	temp[4]=m[1];
	temp[5]=m[5];
	temp[6]=m[9];
	temp[7]=0.0f;
	temp[8]=m[2];
	temp[9]=m[6];
	temp[10]=m[10];
	temp[11]=0.0f;
	temp[15]=1.0f;
	temp[12]=-(m[12]*m[0])-(m[13]*m[1])-(m[14]*m[2]);
	temp[13]=-(m[12]*m[4])-(m[13]*m[5])-(m[14]*m[6]);
	temp[14]=-(m[12]*m[8])-(m[13]*m[9])-(m[14]*m[10]);
	memcpy(m, temp, sizeof(float)*16);
}

// -------------------------------------------------------------- //
// gComputeNormal:
// -------------------------------------------------------------- //
inline static void gComputeNormal(float LastVectors[3][3], float* pNormal)
{
float	a[3];// = {0.0f, 0.0f, 0.0f};
float	b[3];// = {0.0f, 0.0f, 0.0f};
float	result[3];// = {0.0f, 0.0f, 0.0f};
float	length;

	b[0]=(LastVectors[0][0] - LastVectors[1][0]);
	b[1]=(LastVectors[0][1] - LastVectors[1][1]);
	b[2]=(LastVectors[0][2] - LastVectors[1][2]);

	a[0]=-(LastVectors[1][0] - LastVectors[2][0]);
	a[1]=-(LastVectors[1][1] - LastVectors[2][1]);
	a[2]=-(LastVectors[1][2] - LastVectors[2][2]);

	// a x b
	result[0]= ((a[1]*b[2]) - (b[1]*a[2]));
	result[1]= ((a[2]*b[0]) - (b[2]*a[0]));
	result[2]= ((a[0]*b[1]) - (b[0]*a[1]));

	// calculate the length of the normal
	length=gGetLength(result);

	// normalize and specify the normal
	if (length!=0.0f)
	{
		pNormal[0]=result[0]/length;
		pNormal[1]=result[1]/length;
		pNormal[2]=result[2]/length;
	}
}

inline void gComputeTangentBasis(float LastVectors[3][3], float LastTextures[3][2], float *tangent, float *binormal)
{
float cp[3];
float e0[3]={LastVectors[1][0]-LastVectors[0][0], LastTextures[1][0]-LastTextures[0][0], LastTextures[1][1]-LastTextures[0][1]};
float e1[3]={LastVectors[2][0]-LastVectors[0][0], LastTextures[2][0]-LastTextures[0][0], LastTextures[2][1]-LastTextures[0][1]};

	cp[0]=e0[1]*e1[2]-e0[2]*e1[1];
	cp[1]=e0[2]*e1[0]-e0[0]*e1[2];
	cp[2]=e0[0]*e1[1]-e0[1]*e1[0];

	if(fabs(cp[0])>0.00001f)
	{
		tangent[0]=-cp[1]/cp[0];
		binormal[0]=-cp[2]/cp[0];
	}

	e0[0]=LastVectors[1][1]-LastVectors[0][1];
	e1[0]=LastVectors[2][1]-LastVectors[0][1];

	cp[0]=e0[1]*e1[2]-e0[2]*e1[1];
	cp[1]=e0[2]*e1[0]-e0[0]*e1[2];
	cp[2]=e0[0]*e1[1]-e0[1]*e1[0];

	if(fabs(cp[0])>0.00001f)
	{
		tangent[1]=-cp[1]/cp[0];
		binormal[1]=-cp[2]/cp[0];
	}

	e0[0]=LastVectors[1][2]-LastVectors[0][2];
	e1[0]=LastVectors[2][2]-LastVectors[0][2];

	cp[0]=e0[1]*e1[2]-e0[2]*e1[1];
	cp[1]=e0[2]*e1[0]-e0[0]*e1[2];
	cp[2]=e0[0]*e1[1]-e0[1]*e1[0];

	if(fabs(cp[0])>0.00001f)
	{
		tangent[2]=-cp[1]/cp[0];
		binormal[2]=-cp[2]/cp[0];
	}
}


// -------------------------------------------------------------- //
// gTransformVector: V'=V*M
// -------------------------------------------------------------- //
inline static void gTransformVector(float* mat, float* vec)
{
static float	vector[3];
	vector[0]=vec[0]*mat[0]+vec[1]*mat[1]+vec[2]*mat[2]+mat[3];// vector[4]=1.0
	vector[1]=vec[0]*mat[4]+vec[1]*mat[5]+vec[2]*mat[6]+mat[7];
	vector[2]=vec[0]*mat[8]+vec[1]*mat[9]+vec[2]*mat[10]+mat[11];
	vec[0] = vector[0];
	vec[1] = vector[1];
	vec[2] = vector[2];
}

#endif