/**************************************************************

GLBasic .ddd export plugin for AC3D  - 

Version 1.0 by Gernot Frisch (Gernot.Frisch@Dream-D-Sign.de)


This plugin allows you to export your model in the GLBasic .ddd
file format. If you select objects/groups, only the selected
surfaces get exported. Non triangluar surfaces are trianglulated
the way the SV-plugin does.
The .ddd file format is a very compact format. It's binary, so
porting to little endians might become a problem. One .ddd file
always uses the same texture. No animations are exported so far,
but when it's possible in AC3D, I'll extend it.
Huge models should be created carefully, since they get mapped to
a 256x256x256 grid for good compression ratios.

If you run into trouble with the filter, please feel free to contact
me:
Gernot.Frisch@Dream-D-Sign.de or visit www.GLBasic.com

**************************************************************/

// #include <stdio.h>
#include "../ac_plugin.h"
#include "MeshObject.h"
#include "GFFileSystem.h"


// Color value of Material ID
static long ddd_get_material_color_rgb(int col)
{
	material_t *m = ac_palette_get_material(col);
	long rgb;
	rgb = ac_material_index_to_rgb(col);
	return rgb;
}

/*// Data
struct POLY_NODE // KEEP this order!!
{
float			pos[3];
float			normal[3];
float			tex[2];
float			bump_light[3]; // offes vertex glColor
float			tangent[3];
float			binormal[3];
unsigned long	col;
};
*/

// Triangles of surface
static void ddd_output_triangle(POLY_FRAME *pframe, List *vertices, Surface *s)
{
	SVertex *p1;
	POLY_NODE	node;
	POLY_NODE	node_new_tri;
	node_new_tri.col = COL_NEW_TRI;
	if (s->numvert == 3)
	{
		pframe->nodes.Push(node_new_tri);

		node.normal[0] = s->normal.x;
		node.normal[1] = s->normal.y;
		node.normal[2] = s->normal.z;
		node.col = ddd_get_material_color_rgb(s->col);

		for (long n=0; n<3; n++)
		{
			switch(n)
			{
			case 0: p1 = (SVertex*)(s->vertlist->data); break;
			case 1: p1 = (SVertex*)(s->vertlist->next->data); break;
			case 2: p1 = (SVertex*)(s->vertlist->next->next->data); break;
			}
			node.pos[0] = p1->v->x;
			node.pos[1] = p1->v->y;
			node.pos[2] = p1->v->z;
			node.tex[0] = p1->tx;
			node.tex[1] = - p1->ty;

			pframe->nodes.Push(node);
		}
	}
}


// Export a surface regardless if triangles or poly - needs optimization!
static void ddd_output_surfaces(POLY_FRAME *pframe, List *vertices, List *surfaces)
/** run though each surface and see if it needs triangulating
if it needs triangulating then find the triangulations and store the new surfaces
otherwise store a reference to the surface and output the triangle
Surfaces with < 3 vertices will be ignored.
**/
{
	int	stype;
	List *p, *ps;
	List *triangs = NULL;  /** a list of lists of triangles **/
	List *trisurfaces = NULL; /** list of surfaces that are triangles (didn't need triangluating) **/
	int numtri = 0;

	/** triangulate each surface and store a list of the triangles **/
	/** count up the number of triangles **/
	for (p = surfaces; p != NULL; p = p->next)
	{
		Surface *s = (Surface *)p->data;
		List *slist;

		stype = surface_get_type(s);
		if (stype != SURFACE_POLYGON)
		{
			/** ignore any surface other than polygons **/
		}
		if (s->numvert == 3)
		{
			/** surface is a triangle - remember it for later **/
			list_add_item(&trisurfaces, s);
			numtri++;
		}
		else 
			if (s->numvert > 3)
			{
				/** need to triangluate surfaces **/
				slist = (List *)surface_get_triangulations(s);

				if (slist != NULL)
				{
					numtri += list_count(slist);
					list_add_item(&triangs, slist);
				}           
			}
	}

	/** go through the triangle list and output each **/
	for (p = triangs; p != NULL; p = p->next)
	{
		List *tlist = (List *)p->data; /** get the list of triangle surfaces **/

		/** output each triangle then free the surface **/
		for (ps = tlist; ps != NULL; ps = ps->next)
		{
			Surface *s = (Surface *)ps->data;
			ddd_output_triangle(pframe, vertices, s);
			surface_free(s); /** free the triangle surface **/
		}
		list_free(&tlist);  /** free the triangle list **/
	}
	list_free(&triangs);  /** free the list of triangle lists **/

	/** output the surfaces that didn't need triangluating **/
	for (p = trisurfaces; p != NULL; p = p->next)
	{
		Surface *s = (Surface *)p->data;
		ddd_output_triangle(pframe, vertices, s);
	}
	list_free(&trisurfaces);
}

// Output a complete object
static void ddd_output_object(POLY_FRAME* pframe, ACObject *ob)
{
	int numvert, numsurf, numkids;
	List *vertices, *surfaces, *kids;
	List *p;

	ac_object_get_contents(ob, &numvert, &numsurf, &numkids,
		&vertices, &surfaces, &kids); 

	if (numvert > 0)
	{
		ddd_output_surfaces(pframe, vertices, surfaces);
	}
	for (p = kids; p != NULL; p = p->next)
		ddd_output_object(pframe, (ACObject *)p->data); // call recursively for child objects
}

static void do_ddd_save()
{
	char *fname;
	char *filter[] = { "ddd files", ".ddd", "All files", "*", NULL };
	fname = ac_get_save_filename("Export GLBasic .ddd file", filter);
	if ( (fname == NULL) || (*fname =='\0') )
		return;


List *templist;
ACObject *newworld;
POLY_FRAME	frame;
MeshObject	mesh;

	templist = (List *)clone_selected(FALSE);
	if (templist)
	{
		display_message("Exporting selection to .ddd file...");
		newworld = (ACObject *)new_object(OBJECT_WORLD);
		object_add_children(newworld, templist);

		ddd_output_object(&frame, newworld);
		object_delete(newworld);
		list_free(&templist);
		display_message(" ");
	}
	else
	{
		display_message("Exporting all to .ddd file...");
		newworld = ac_get_world();
		ddd_output_object(&frame, newworld);
	}

	mesh.m_Frames.Push(frame);
	display_message("Compressing...\n");
	mesh.Compress();
	mesh.Save(fname);
	display_message(" ");
}



/*
typedef struct prefspec
{
    char *name;
    int type; // PREF_
    void *addr; // pointer to current value
} PrefSpec;

// Note that string values must be allocated with  STRING();

Prototype PrefSpec *ac_prefs_get_spec(char *name);  // returns NULL if not found 


// best to use these functions to access prefs:

Prototype int ac_prefs_list();  // list prefs names with types  - returns total number of prefs
Prototype int ac_prefs_get_int(char *name);
Prototype Boolean ac_prefs_get_boolean(char *name);
Prototype double ac_prefs_get_double(char *name);
Prototype int ac_prefs_get_hex(char *name);
Prototype char *ac_prefs_get_string(char *name);
Prototype char *ac_prefs_get_key(char *name);
Prototype void ac_tcl_link_pref(PrefSpec *p); // make this preference available in tcl as pref_<p->name>
Prototype void prefs_append_prefspec(PrefSpec *n); // add a new pref to the main list
*/



/***** hook functions that the AC3D plugin loader looks for *******/

extern "C" __declspec(dllexport) int AC3DPluginInit()
{
	double ver = ac_get_version_number();

	ac_add_command("ddd_export_plugin", do_ddd_save);
	ac_add_export_menu_item("GLBasic ddd", "ac3d ddd_export_plugin");
	return(0);
}


extern "C" __declspec(dllexport) int AC3DPluginExit()
{
	return(0);
}

extern "C" __declspec(dllexport) char *AC3DPluginAbout()
{
	return("GLBasic .ddd Export Plugin - Version 1.0 - by Gernot Frisch");
}




